#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from decentra_network.config import TEMP_ACCOUNTS_PATH
from decentra_network.lib.config_system import get_config


def SaveAccounts(the_accounts, custom_TEMP_ACCOUNTS_PATH=None):
    """
    Saves the accounts to the TEMP_ACCOUNTS_PATH.
    """

    the_TEMP_ACCOUNTS_PATH = (
        TEMP_ACCOUNTS_PATH
        if custom_TEMP_ACCOUNTS_PATH is None
        else custom_TEMP_ACCOUNTS_PATH
    )
    os.chdir(get_config()["main_folder"])
    # turn account list to a json array (use dump_json for objects)
    the_accounts_json = {
        account.Address: account.dump_json() for account in the_accounts
    }

    with open(the_TEMP_ACCOUNTS_PATH, "w") as block_file:
        json.dump(the_accounts_json, block_file)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

from decentra_network.accounts.get_accounts import GetAccounts
from decentra_network.accounts.save_accounts import SaveAccounts
from decentra_network.blockchain.block.save_block import SaveBlock
from decentra_network.blockchain.block.blocks_hash import GetBlockshash
from decentra_network.blockchain.block.blocks_hash import SaveBlockshash
from decentra_network.blockchain.block.blocks_hash import GetBlockshash_part
from decentra_network.blockchain.block.blocks_hash import SaveBlockshash_part
from decentra_network.config import BLOCKS_PATH
from decentra_network.wallet.ellipticcurve.wallet_import import wallet_import


def SaveBlockstoBlockchainDB(
    block,
    custom_BLOCKS_PATH=None,
    custom_TEMP_ACCOUNTS_PATH=None,
    custom_TEMP_BLOCKSHASH_PATH=None,
    custom_TEMP_BLOCKSHASH_PART_PATH=None,
):
    """
    Adds the block to the blockchain database
    at BLOCKS_PATH.
    """

    my_public_key = "".join(
        [
            l.strip()
            for l in wallet_import(-1, 0).splitlines()
            if l and not l.startswith("-----")
        ]
    )
    my_address = wallet_import(-1, 3)
    our_tx = any(
        (validated_transaction.fromUser == my_public_key)
        or (validated_transaction.toUser == my_address)
        for validated_transaction in block.validating_list
    )

    # If the block is our transaction, then add it to the blockchain database.
    if our_tx:
        the_BLOCKS_PATH = (
            BLOCKS_PATH if custom_BLOCKS_PATH is None else custom_BLOCKS_PATH
        )
        SaveBlock(block, (the_BLOCKS_PATH + str(block.sequance_number) + ".block.json"))
        SaveAccounts(
            GetAccounts(custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH),
            (the_BLOCKS_PATH + str(block.sequance_number) + ".accounts.json"),
        )
        SaveBlockshash(
            GetBlockshash(custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH),
            (the_BLOCKS_PATH + str(block.sequance_number) + ".blockshash.json"),
        )
        SaveBlockshash_part(
            GetBlockshash_part(
                custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH
            ),
            (the_BLOCKS_PATH + str(block.sequance_number) + ".blockshashpart.json"),
        )

    else:
        False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.lib.log import get_logger

logger = get_logger("TRANSACTIONS")


def Check_Len(block, transaction):
    """
    Check if the transaction lenght is valid
    """

    if (block.max_data_size / block.max_tx_number) >= len(transaction.data):
        logger.info("Data len is true")
    else:
        return False

    if len(transaction.fromUser) == 120:
        logger.info("The from user is correct")
    else:
        return False

    if len(transaction.toUser) <= 40:
        logger.info("The to user is correct")
    else:
        return False

    decimal_amount = len(str(block.transaction_fee).split(".")[1])

    if len(str(transaction.amount).split(".")[1]) <= decimal_amount:
        logger.info("The decimal amount of transaction.amount is true.")
    else:
        return False

    if len(str(transaction.transaction_fee).split(".")[1]) <= decimal_amount:
        logger.info("The decimal amount of transaction.transaction_fee is true.")
    else:
        return False

    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from decentra_network.lib.log import get_logger
from decentra_network.wallet.ellipticcurve.ecdsa import Ecdsa
from decentra_network.wallet.ellipticcurve.publicKey import PublicKey
from decentra_network.wallet.ellipticcurve.signature import Signature

logger = get_logger("TRANSACTIONS")


def Check_Sign(transaction):
    """
    Check if the transaction signature is valid
    """

    if Ecdsa.verify(
        (
            str(transaction.sequance_number)
            + str(transaction.fromUser)
            + str(transaction.toUser)
            + str(transaction.data)
            + str(transaction.amount)
            + str(transaction.transaction_fee)
            + str(transaction.transaction_time)
        ),
        Signature.fromBase64(transaction.signature),
        PublicKey.fromPem(transaction.fromUser),
    ):
        logger.info("The signature is valid")
    else:
        return False

    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from decentra_network.accounts.account import Account
from decentra_network.config import TEMP_ACCOUNTS_PATH
from decentra_network.lib.config_system import get_config


def GetAccounts(custom_TEMP_ACCOUNTS_PATH=None):
    """
    Returns the accounts from TEMP_ACCOUNTS_PATH.
    """

    the_TEMP_ACCOUNTS_PATH = (
        TEMP_ACCOUNTS_PATH
        if custom_TEMP_ACCOUNTS_PATH is None
        else custom_TEMP_ACCOUNTS_PATH
    )

    os.chdir(get_config()["main_folder"])
    if not os.path.exists(the_TEMP_ACCOUNTS_PATH):
        return []
    with open(the_TEMP_ACCOUNTS_PATH, "r") as block_file:
        the_accounts_json = json.load(block_file)

    return [
        Account.load_json(the_accounts_json[account_json])
        for account_json in the_accounts_json
    ]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

from decentra_network.lib.log import get_logger
from decentra_network.lib.settings_system import the_settings

logger = get_logger("Safety")


def safety_check(interface=None, timeout=None):
    logger.info("Checking safety")
    try:
        from pywall import pywall

        the_pywall = pywall()
        if interface is not None:
            the_pywall.iface = interface
        if timeout is not None:
            the_pywall.timeout = timeout
        control = the_pywall.control()
        if control is not None:
            if control:
                logger.info("NOT Safe")
                exit()
            else:
                logger.info("Safe")
        elif the_settings()["debug_mode"]:
            logger.info("Control check is none but passing because of debug mode")
        else:
            logger.info("NOT Safe (Control check is None)")
            exit()
    except ImportError:
        logger.info("Passing safety check (no pywall)")
